# -*- encoding: utf-8 -*-

import httpx

from django.conf import settings
from django.utils import timezone
from asgiref.sync import sync_to_async
from django.shortcuts import redirect
from django.http import JsonResponse, HttpResponse
from django.views.generic import TemplateView

from apps.home.models import r_Server
from apps.authentication.models import DiscordUser, ServerMember


class IndexView(TemplateView):
    """
    View for the Index page.
    """

    template_name = "home/index.html"


@sync_to_async
def get_user_access_token(user: DiscordUser) -> str:
    return user.access_token

@sync_to_async
def is_user_authenticated(user: DiscordUser) -> bool:
    return user.is_authenticated

async def delete_server(server_id: int):
    # member will be auto deleted via CASCADE, so no need to handle that

    try:
        server = await r_Server.objects.aget(id=server_id)
        await server.adelete()
    except r_Server.DoesNotExist:
        pass

async def setup_server(user: DiscordUser, data: dict) -> dict | None:
    is_owner = data["owner"]
    permissions = data["permissions"]
    admin_perm = 1 << 3

    # Ignore servers where the user isn't an administrator or owner
    if not ((int(permissions) & admin_perm) == admin_perm or is_owner):
        await delete_server(data["id"])
        return

    server = await r_Server.objects.aget_or_create(
        id=data["id"],
        name=data["name"],
        icon_hash=data["icon"]
    )
 
    await ServerMember.objects.aupdate_or_create(
        user=user,
        server=server[0],
        defaults={
            "permissions": permissions,
            "is_owner": is_owner
        }
    )

    return data


async def guilds(request):
    if not await is_user_authenticated(request.user):
        return redirect("/oauth2/login")

    access_token = await get_user_access_token(request.user)

    async with httpx.AsyncClient() as client:
        response = await client.get(
            url=f"{settings.DISCORD_API_URL}/users/@me/guilds",
            headers={"Authorization": f"Bearer {access_token}"}
        )
        guild_data = response.json()

    # guilds where the user either administrates or owns
    cleaned_guild_data = []

    for item in guild_data:
        cleaned_data = await setup_server(request.user, item)
        if cleaned_data:
            cleaned_guild_data.append(cleaned_data)


    return JsonResponse(guild_data, safe=False)